package com.example.liz.navyproject;

import android.app.Activity;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.hardware.Camera;
import android.hardware.camera2.CameraManager;
import android.media.MediaRecorder;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Environment;
import android.preference.PreferenceManager;
import android.provider.MediaStore;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.View;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import com.perforce.p4java.client.IClient;
import com.perforce.p4java.core.ChangelistStatus;
import com.perforce.p4java.core.IChangelist;
import com.perforce.p4java.core.IMapEntry;
import com.perforce.p4java.core.file.FileSpecBuilder;
import com.perforce.p4java.core.file.FileSpecOpStatus;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.impl.generic.client.ClientView;
import com.perforce.p4java.impl.generic.core.Changelist;
import com.perforce.p4java.impl.generic.core.file.FileSpec;
import com.perforce.p4java.impl.mapbased.client.Client;
import com.perforce.p4java.impl.mapbased.server.Server;
import com.perforce.p4java.option.changelist.SubmitOptions;
import com.perforce.p4java.option.client.AddFilesOptions;
import com.perforce.p4java.option.client.EditFilesOptions;
import com.perforce.p4java.server.IOptionsServer;
import com.perforce.p4java.server.IServerInfo;
import com.perforce.p4java.server.ServerFactory;

import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.List;

public class MainActivity extends AppCompatActivity {

    TextView tv;
    TextView tv2;
    Button button;
    Button resetButton;
    ProgressBar pb;
    CheckBox checkBoxPic;
    CheckBox checkBoxAudio;
    private static final int REQUEST_CODE = 1;
    private Uri imageUri;
    private Uri audioUri;
    private String timestamp;
    private String filename = "pic" + ".jpg";
    private MediaRecorder mRecorder;
    private String audioFilename = "audio" + ".3gp";
    final private String clientRoot = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES) + "/NavyProject";
    private boolean audioSubmitted = false;
    private boolean picSubmitted = false;
    String p4user;
    String p4port;
    String p4password;
    String p4path;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        File f = new File(clientRoot);
        if (!f.isDirectory()) {
            f.mkdir();
        }

        tv = (TextView) findViewById(R.id.textView);
        tv2 = (TextView) findViewById(R.id.textView2);
        resetButton = (Button) findViewById(R.id.resetButton);
        pb = (ProgressBar) findViewById(R.id.progressBar);
        checkBoxPic = (CheckBox) findViewById(R.id.checkBox);
        checkBoxAudio = (CheckBox) findViewById(R.id.checkBox2);

        final FloatingActionButton fab_pic = (FloatingActionButton) findViewById(R.id.fab_pic);
        final FloatingActionButton fab_speak = (FloatingActionButton) findViewById(R.id.fab_speak);
        final FloatingActionButton fab_speak_save = (FloatingActionButton) findViewById(R.id.fab_speak_save);
        final FloatingActionButton fab = (FloatingActionButton) findViewById(R.id.fab);

        resetButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                reset();
                resetButton.setVisibility(View.INVISIBLE);
            }
        });

        fab_pic.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (timestamp == null) {
                    timestamp = String.valueOf(new Date().getTime() / 1000);
                }
                Snackbar.make(view, "Take Picture", Snackbar.LENGTH_LONG)
                        .setAction("Action", null).show();
                Intent intent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
                File photo = new File(clientRoot, timestamp + "-" + filename);
                intent.putExtra(MediaStore.EXTRA_OUTPUT, Uri.fromFile(photo));
                imageUri = Uri.fromFile(photo);
                startActivityForResult(intent, REQUEST_CODE);
            }
        });

        fab_speak.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (timestamp == null) {
                    timestamp = String.valueOf(new Date().getTime() / 1000);
                }
                Snackbar.make(view, "Record Voice", Snackbar.LENGTH_LONG)
                        .setAction("Action", null).show();
                fab_speak.setVisibility(View.INVISIBLE);
                fab_speak_save.setVisibility(View.VISIBLE);
                tv.setText("Recording...");
                startRecording();
            }
        });

        fab_speak_save.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                stopRecording();
                tv.setText("Audio File Saved");
                tv2.setText(timestamp + "-" + audioFilename);
                fab_speak_save.setVisibility(View.INVISIBLE);
                fab_speak.setVisibility(View.VISIBLE);
            }
        });

        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                SharedPreferences settings = getSharedPreferences(PreferenceActivity.P4PREFS, Context.MODE_PRIVATE);
                p4port = settings.getString(PreferenceActivity.P4PORT, null);
                p4user = settings.getString(PreferenceActivity.P4USER, null);
                p4password = settings.getString(PreferenceActivity.P4PASSSWD, null);
                p4path = settings.getString(PreferenceActivity.P4PATH, "//depot/navy-project");

                if (p4port == null || p4user == null) {
                    Snackbar.make(view, "Go to Settings to add P4PORT and P4USER", Snackbar.LENGTH_LONG)
                            .setAction("Action", null).show();
                }

                if (imageUri != null || audioUri != null ) {
                    pb.setVisibility(View.VISIBLE);
                    submitFiles();
                    Snackbar.make(view, "Preparing to submit.", Snackbar.LENGTH_LONG)
                            .setAction("Action", null).show();
                } else {
                    Snackbar.make(view, "Take a picture or record audio first.", Snackbar.LENGTH_LONG)
                            .setAction("Action", null).show();
                }
            }
        });
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == Activity.RESULT_OK) {
            ImageView imageView = (ImageView) findViewById(R.id.imageView);
            Uri selectedImage = imageUri;
            getContentResolver().notifyChange(selectedImage, null);
            ContentResolver cr = getContentResolver();
            Bitmap bitmap;
            try {
                bitmap = MediaStore.Images.Media.getBitmap(cr, selectedImage);
                int h = (int) (bitmap.getHeight() * (512.0 / bitmap.getWidth()));
                Bitmap scaled = Bitmap.createScaledBitmap(bitmap, 512, h, true);
                imageView.setImageBitmap(scaled);
                Toast.makeText(this, selectedImage.toString(), Toast.LENGTH_LONG).show();
                tv.setText("Picture Saved");
            } catch (Exception e) {

            }
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        if (id == R.id.home_settings) {
            return true;
        } else if (id == R.id.action_settings) {
            Intent intent = new Intent(this, PreferenceActivity.class);
            startActivity(intent);
            return true;
        } else if (id == R.id.info_settings) {
            getInfo();
        }

        return super.onOptionsItemSelected(item);
    }

    private boolean checkCameraHardware(Context context) {
        if (context.getPackageManager().hasSystemFeature(PackageManager.FEATURE_CAMERA)) {
            return true;
        } else {
            return false;
        }
    }

    private void startRecording() {
        mRecorder = new MediaRecorder();
        mRecorder.setAudioSource(MediaRecorder.AudioSource.MIC);
        mRecorder.setOutputFormat(MediaRecorder.OutputFormat.THREE_GPP);
        File audio = new File(clientRoot, timestamp + "-" + audioFilename);
        audioUri = Uri.fromFile(audio);
        mRecorder.setOutputFile(audioUri.getPath());
        mRecorder.setAudioEncoder(MediaRecorder.AudioEncoder.AMR_NB);
        try {
            mRecorder.prepare();
        } catch (IOException e) {
            Toast.makeText(this, e.toString(), Toast.LENGTH_LONG);
        }
        mRecorder.start();
    }

    private void stopRecording() {
        mRecorder.stop();
        mRecorder.release();
        mRecorder = null;
    }

    protected void getInfo() {
        P4Task p4info = new P4Task();
        p4info.execute("info");
    }

    protected void submitFiles() {
        P4Task p4submit = new P4Task();
        p4submit.execute("submit");
    }

    protected void reset() {
        imageUri = null;
        audioUri = null;
        timestamp = null;
        ImageView imageView = (ImageView) findViewById(R.id.imageView);
        imageView.setImageResource(android.R.color.transparent);
        tv2.setText("No Audio Recorded");
        tv.setText("");
        checkBoxAudio.setVisibility(View.INVISIBLE);
        checkBoxAudio.setChecked(false);
        checkBoxPic.setVisibility(View.INVISIBLE);
        checkBoxPic.setChecked(false);
        audioSubmitted = false;
        picSubmitted = false;
    }

    private class P4Task extends AsyncTask<String, String, String> {
        @Override
        protected String doInBackground(String... params) {
            String msg = "test";
            IOptionsServer server = null;
            if (p4port ==  null) {
                p4port = "perforce:1666";
            }
            try {
                server = ServerFactory.getOptionsServer("p4java://" + p4port, null);
                server.setUserName(p4user);
                server.connect();
                if (p4password != null) {
                    server.login(p4password);
                }
            } catch (Exception e) {
                msg = e.toString();
            }

            if (params[0] == "submit") {
                try {
                    IClient client = new Client();
                    client.setName("temp-client");
                    client.setOwnerName(p4user);
                    client.setRoot(clientRoot);
                    client.setServer(server);
                    // trim ending slash if exists
                    if ((p4path.charAt(p4path.length()-1)) == '/') {
                        p4path = p4path.substring(0, p4path.length()-2);
                    }
                    String depotPath = p4path + "/" + timestamp + "/";
                    String description = "Timestamp: " + timestamp + "\nSubmitting files:\n ";

                    // Set the client for the current server
                    server.setCurrentClient(client);

                    // Create client view mapping
                    ClientView.ClientViewMapping clientViewMapping = new ClientView.ClientViewMapping();
                    clientViewMapping.setLeft(depotPath + "...");
                    clientViewMapping.setRight("//" + client.getName() + "/...");
                    clientViewMapping.setType(IMapEntry.EntryType.INCLUDE);

                    //Create client view
                    ClientView clientView = new ClientView();
                    clientView.addEntry(clientViewMapping);
                    client.setClientView(clientView);

                    server.createClient(client);

                    Changelist changeListImpl = new Changelist(
                            IChangelist.UNKNOWN,
                            client.getName(),
                            server.getUserName(),
                            ChangelistStatus.NEW,
                            new Date(),
                            description,
                            false,
                            (Server) server);

                    IChangelist changelist = client.createChangelist(changeListImpl);
                    AddFilesOptions addFilesOptions = new AddFilesOptions().setChangelistId(changelist.getId());
                    String filepath = depotPath + timestamp + "-";

                    if (imageUri != null) {
                        description = description + timestamp + "-" + filename;
                        changelist.setDescription(description);
                        if (!picSubmitted) {
                            client.addFiles(FileSpecBuilder.makeFileSpecList(filepath + filename),
                                    false, changelist.getId(), "binary", false);
                        } else {
                            client.editFiles(FileSpecBuilder.makeFileSpecList(filepath + filename), false, false,
                                    changelist.getId(), "binary");
                        }
                    } else {
                        // TODO: need to decide if image is required
                    }
                    if (audioUri != null) {
                        description = description + "\n" + timestamp + "-" + audioFilename;
                        changelist.setDescription(description);
                        if (!audioSubmitted) {
                            client.addFiles(FileSpecBuilder.makeFileSpecList(filepath + audioFilename),
                                    false, changelist.getId(), "binary", false);
                        } else {
                            client.editFiles(FileSpecBuilder.makeFileSpecList(filepath + audioFilename), false, false,
                                    changelist.getId(), "binary");
                        }
                    } else {
                        // TODO: need to decide if audio is required
                    }
                    changelist.update();
                    changelist.refresh();

                    List<IFileSpec> submitFiles = changelist.submit(null);
                    if (submitFiles != null) {
                        for (IFileSpec filespec : submitFiles) {
                            if (filespec != null) {
                                if (filespec.getOpStatus() == FileSpecOpStatus.VALID) {
                                    msg = "\nSubmitted: " + filespec.getDepotPathString();
                                } else if (filespec.getOpStatus() == FileSpecOpStatus.INFO) {
                                    msg = "INFO: " + filespec.getStatusMessage() +
                                           "\nFILES:" + changelist.getFiles(true).toString();
                                } else if (filespec.getOpStatus() == FileSpecOpStatus.ERROR) {
                                    msg = "ERROR: " + filespec.getStatusMessage();
                                }
                            }
                        }
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
                return msg;
            } else if (params[0] == "info") {
                try {
                    IServerInfo infoServer = server.getServerInfo();
                    msg = "Version: " + infoServer.getServerVersion() + "\nP4PORT: " + infoServer.getServerAddress() +
                            "\nP4USER: " + infoServer.getUserName();
                } catch (Exception e) {
                    msg = e.toString();
                }
            }
            return msg;
        }

        @Override
        protected void onPostExecute(String s) {
            pb.setVisibility(View.INVISIBLE);
            Context context = getApplicationContext();
            Toast.makeText(context, s, Toast.LENGTH_LONG).show();
            if (s.contains("Submitted")) {
                resetButton.setVisibility(View.VISIBLE);
            }
            if (s.contains("audio")) {
                checkBoxAudio.setVisibility(View.VISIBLE);
                checkBoxAudio.setChecked(true);
                audioSubmitted = true;
            }
            if (s.contains("pic")) {
                checkBoxPic.setVisibility(View.VISIBLE);
                checkBoxPic.setChecked(true);
                picSubmitted = true;
            }
            tv.setText(s);

        }
    }

}
